@@ -88,7 +88,7 @@ struct _EmpathyAccountSettingsPriv
   gboolean password_retrieved;
   gboolean password_requested;
 
-  /* Parameter name (gchar *) -> parameter value (GValue) */
+  /* Parameter name (gchar *) -> parameter value (GVariant) */
   GHashTable *parameters;
   /* Keys are parameter names from the hash above (gchar *).
    * Values are regular expresions that should match corresponding parameter
@@ -123,7 +123,7 @@ empathy_account_settings_init (EmpathyAccountSettings *obj)
   priv->account_manager = tp_account_manager_dup ();
 
   priv->parameters = g_hash_table_new_full (g_str_hash, g_str_equal,
-    g_free, (GDestroyNotify) tp_g_value_slice_free);
+    g_free, (GDestroyNotify) g_variant_unref);
 
   priv->param_regexps = g_hash_table_new_full (g_str_hash, g_str_equal,
     g_free, (GDestroyNotify) g_regex_unref);
@@ -472,7 +472,7 @@ empathy_account_settings_migrate_password_cb (GObject *source,
   GError *error = NULL;
   EmpathyAccountSettings *self = user_data;
   EmpathyAccountSettingsPriv *priv = GET_PRIV (self);
-  GHashTable *empty;
+  GVariantBuilder *builder;
   const gchar *unset[] = { "password", NULL };
 
   if (!empathy_keyring_set_account_password_finish (account, result, &error))
@@ -483,26 +483,32 @@ empathy_account_settings_migrate_password_cb (GObject *source,
     }
 
   /* Now clear the password MC has stored. */
-  empty = tp_asv_new (NULL, NULL);
-  tp_account_update_parameters_async (priv->account,
-      empty, unset, NULL, NULL);
+  builder = g_variant_builder_new (G_VARIANT_TYPE_VARDICT);
+
+  tp_account_update_parameters_vardict_async (priv->account,
+      g_variant_builder_end (builder), unset, NULL, NULL);
 
   g_hash_table_remove (priv->parameters, "password");
-
-  g_hash_table_unref (empty);
 }
 
+static GVariant * empathy_account_settings_dup (
+    EmpathyAccountSettings *settings,
+    const gchar *param);
+
 static void
 empathy_account_settings_try_migrating_password (EmpathyAccountSettings *self)
 {
   EmpathyAccountSettingsPriv *priv = GET_PRIV (self);
-  const GValue *v;
+  GVariant *v;
   const gchar *password;
 
-  if (!priv->supports_sasl || empathy_account_settings_get (
-          self, "password") == NULL)
+  v = empathy_account_settings_dup (self, "password");
+  if (v == NULL)
     return;
 
+  if (!priv->supports_sasl)
+    goto out;
+
   /* mission-control still has our password, although the CM
    * supports SASL. Let's try migrating it. */
 
@@ -510,16 +516,14 @@ empathy_account_settings_try_migrating_password (EmpathyAccountSettings *self)
       "keyring ourselves for account %s",
       tp_account_get_path_suffix (priv->account));
 
-  v = empathy_account_settings_get (self, "password");
-
   /* I can't imagine why this would fail. */
-  if (!G_VALUE_HOLDS_STRING (v))
-    return;
+  if (!g_variant_is_of_type (v, G_VARIANT_TYPE_STRING))
+    goto out;
 
-  password = g_value_get_string (v);
+  password = g_variant_get_string (v, NULL);
 
   if (EMP_STR_EMPTY (password))
-    return;
+    goto out;
 
   empathy_keyring_set_account_password_async (priv->account, password,
       empathy_account_settings_migrate_password_cb, self);
@@ -530,13 +534,15 @@ empathy_account_settings_try_migrating_password (EmpathyAccountSettings *self)
 
   priv->password = g_strdup (password);
   priv->password_original = g_strdup (password);
+
+out:
+  g_variant_unref (v);
 }
 
 static void
 empathy_account_settings_check_readyness (EmpathyAccountSettings *self)
 {
   EmpathyAccountSettingsPriv *priv = GET_PRIV (self);
-  const TpConnectionManagerProtocol *tp_protocol;
   GQuark features[] = { TP_PROTOCOL_FEATURE_CORE, 0 };
 
   if (priv->ready)
@@ -573,34 +579,11 @@ empathy_account_settings_check_readyness (EmpathyAccountSettings *self)
       priv->uri_scheme_tel = empathy_account_has_uri_scheme_tel (priv->account);
     }
 
-  tp_protocol = tp_connection_manager_get_protocol (priv->manager,
-    priv->protocol);
-
-  if (tp_protocol == NULL)
-    {
-      tp_clear_object (&priv->manager);
-      return;
-    }
-
-  if (priv->required_params == NULL)
-    {
-      TpConnectionManagerParam *cur;
-
-      for (cur = tp_protocol->params; cur->name != NULL; cur++)
-        {
-          if (tp_connection_manager_param_is_required (cur))
-            {
-              priv->required_params = g_list_append (priv->required_params,
-                                                     g_strdup (cur->name));
-            }
-        }
-    }
-
   if (priv->protocol_obj == NULL)
     {
       priv->protocol_obj = g_object_ref (
           tp_connection_manager_get_protocol_object (priv->manager,
-              priv->protocol));
+            priv->protocol));
     }
 
   if (!tp_proxy_is_prepared (priv->protocol_obj, TP_PROTOCOL_FEATURE_CORE)
@@ -621,6 +604,26 @@ empathy_account_settings_check_readyness (EmpathyAccountSettings *self)
         }
     }
 
+  if (priv->required_params == NULL)
+    {
+      GList *params, *l;
+
+      params = tp_protocol_dup_params (priv->protocol_obj);
+      for (l = params; l != NULL; l = g_list_next (l))
+        {
+          TpConnectionManagerParam *cur = l->data;
+
+          if (tp_connection_manager_param_is_required (cur))
+            {
+              priv->required_params = g_list_append (priv->required_params,
+                  g_strdup (tp_connection_manager_param_get_name (cur)));
+            }
+        }
+
+       g_list_free_full (params,
+           (GDestroyNotify) tp_connection_manager_param_free);
+    }
+
   /* NOTE: When removing MC migration code, remove this call, and the
    * function it's calling. That's it. */
   empathy_account_settings_try_migrating_password (self);
@@ -693,25 +696,14 @@ empathy_account_settings_new_for_account (TpAccount *account)
       NULL);
 }
 
-TpConnectionManagerParam *
-empathy_account_settings_get_tp_params (EmpathyAccountSettings *settings)
+GList *
+empathy_account_settings_dup_tp_params (EmpathyAccountSettings *settings)
 {
   EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-  const TpConnectionManagerProtocol *tp_protocol;
 
-  g_return_val_if_fail (priv->manager != NULL, NULL);
-  g_return_val_if_fail (priv->protocol != NULL, NULL);
+  g_return_val_if_fail (priv->protocol_obj != NULL, NULL);
 
-  tp_protocol = tp_connection_manager_get_protocol (priv->manager,
-     priv->protocol);
-  if (tp_protocol == NULL)
-    {
-      DEBUG ("Can't retrieve TpConnectionManagerProtocol for protocol '%s'",
-          priv->protocol);
-      return NULL;
-    }
-
-  return tp_protocol->params;
+  return tp_protocol_dup_params (priv->protocol_obj);
 }
 
 gboolean
@@ -804,23 +796,13 @@ empathy_account_settings_is_unset (EmpathyAccountSettings *settings,
   return FALSE;
 }
 
-static TpConnectionManagerParam *
+static const TpConnectionManagerParam *
 empathy_account_settings_get_tp_param (EmpathyAccountSettings *settings,
     const gchar *param)
 {
-  TpConnectionManagerParam *tp_params =
-      empathy_account_settings_get_tp_params (settings);
-  TpConnectionManagerParam *p;
+  EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
 
-  for (p = tp_params; p != NULL && p->name != NULL; p++)
-    {
-      if (tp_strdiff (p->name, param))
-        continue;
-
-      return p;
-    }
-
-  return NULL;
+  return tp_protocol_get_param (priv->protocol_obj, param);
 }
 
 gboolean
@@ -853,61 +835,62 @@ account_settings_remove_from_unset (EmpathyAccountSettings *settings,
     }
 }
 
-const GValue *
-empathy_account_settings_get_default (EmpathyAccountSettings *settings,
+GVariant *
+empathy_account_settings_dup_default (EmpathyAccountSettings *settings,
     const gchar *param)
 {
-  TpConnectionManagerParam *p;
+  const TpConnectionManagerParam *p;
 
   p = empathy_account_settings_get_tp_param (settings, param);
-
-  if (p == NULL || !(p->flags & TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT))
+  if (p == NULL)
     return NULL;
 
-  return &(p->default_value);
+  return tp_connection_manager_param_dup_default_variant (p);
 }
 
 const gchar *
 empathy_account_settings_get_dbus_signature (EmpathyAccountSettings *settings,
     const gchar *param)
 {
-  TpConnectionManagerParam *p;
+  const TpConnectionManagerParam *p;
 
   p = empathy_account_settings_get_tp_param (settings, param);
 
   if (p == NULL)
     return NULL;
 
-  return p->dbus_signature;
+  return tp_connection_manager_param_get_dbus_signature (p);
 }
 
-const GValue *
-empathy_account_settings_get (EmpathyAccountSettings *settings,
+static GVariant *
+empathy_account_settings_dup (EmpathyAccountSettings *settings,
     const gchar *param)
 {
   EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-  const GValue *result = NULL;
+  GVariant *result;
 
   /* Lookup the update parameters we set */
-  result = tp_asv_lookup (priv->parameters, param);
+  result = g_hash_table_lookup (priv->parameters, param);
   if (result != NULL)
-    return result;
+    return g_variant_ref (result);
 
   /* If the parameters isn't unset use the accounts setting if any */
   if (priv->account != NULL
       && !empathy_account_settings_is_unset (settings, param))
     {
-      const GHashTable *parameters;
+      GVariant *parameters;
 
-      parameters = tp_account_get_parameters (priv->account);
-      result = tp_asv_lookup (parameters, param);
+      parameters = tp_account_dup_parameters_vardict (priv->account);
+      result = g_variant_lookup_value (parameters, param, NULL);
+      g_variant_unref (parameters);
 
       if (result != NULL)
+        /* g_variant_lookup_value() is (transfer full) */
         return result;
     }
 
   /* fallback to the default */
-  return empathy_account_settings_get_default (settings, param);
+  return empathy_account_settings_dup_default (settings, param);
 }
 
 void
@@ -951,76 +934,80 @@ empathy_account_settings_discard_changes (EmpathyAccountSettings *settings)
     priv->uri_scheme_tel = FALSE;
 }
 
-const gchar *
-empathy_account_settings_get_string (EmpathyAccountSettings *settings,
+gchar *
+empathy_account_settings_dup_string (EmpathyAccountSettings *settings,
     const gchar *param)
 {
   EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-  const GValue *v;
+  GVariant *v;
+  gchar *result = NULL;
 
   if (!tp_strdiff (param, "password") && priv->supports_sasl)
     {
-      return priv->password;
+      return g_strdup (priv->password);
     }
 
-  v = empathy_account_settings_get (settings, param);
-
-  if (v == NULL || !G_VALUE_HOLDS_STRING (v))
+  v = empathy_account_settings_dup (settings, param);
+  if (v == NULL)
     return NULL;
 
-  return g_value_get_string (v);
+  if (g_variant_is_of_type (v, G_VARIANT_TYPE_STRING))
+    result = g_variant_dup_string (v, NULL);
+
+  g_variant_unref (v);
+  return result;
 }
 
-const gchar * const *
-empathy_account_settings_get_strv (EmpathyAccountSettings *settings,
+GStrv
+empathy_account_settings_dup_strv (EmpathyAccountSettings *settings,
     const gchar *param)
 {
-  const GValue *v;
+  GVariant *v;
+  GStrv result = NULL;
 
-  v = empathy_account_settings_get (settings, param);
-
-  if (v == NULL || !G_VALUE_HOLDS (v, G_TYPE_STRV))
+  v = empathy_account_settings_dup (settings, param);
+  if (v == NULL)
     return NULL;
 
-  return g_value_get_boxed (v);
+  if (g_variant_is_of_type (v, G_VARIANT_TYPE_STRING_ARRAY))
+    result = g_variant_dup_strv (v, NULL);
+
+  g_variant_unref (v);
+  return result;
 }
 
 gint32
 empathy_account_settings_get_int32 (EmpathyAccountSettings *settings,
     const gchar *param)
 {
-  const GValue *v;
+  GVariant *v;
   gint32 ret = 0;
 
-  v = empathy_account_settings_get (settings, param);
-
+  v = empathy_account_settings_dup (settings, param);
   if (v == NULL)
     return 0;
 
-  switch G_VALUE_TYPE (v)
+  if (g_variant_is_of_type (v, G_VARIANT_TYPE_BYTE))
+    ret = g_variant_get_byte (v);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_INT32))
+    ret = g_variant_get_int32 (v);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_UINT32))
+    ret = CLAMP (g_variant_get_uint32 (v), (guint) G_MININT32,
+        G_MAXINT32);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_INT64))
+    ret = CLAMP (g_variant_get_int64 (v), G_MININT32, G_MAXINT32);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_UINT64))
+    ret = CLAMP (g_variant_get_uint64 (v), (guint64) G_MININT32, G_MAXINT32);
+  else
     {
-      case G_TYPE_UCHAR:
-        ret = g_value_get_uchar (v);
-        break;
-      case G_TYPE_INT:
-        ret = g_value_get_int (v);
-        break;
-      case G_TYPE_UINT:
-        ret = CLAMP (g_value_get_uint (v), (guint) G_MININT32,
-            G_MAXINT32);
-        break;
-      case G_TYPE_INT64:
-        ret = CLAMP (g_value_get_int64 (v), G_MININT32, G_MAXINT32);
-        break;
-      case G_TYPE_UINT64:
-        ret = CLAMP (g_value_get_uint64 (v), (guint64) G_MININT32,
-            G_MAXINT32);
-        break;
-      default:
-        ret = 0;
-        break;
+      gchar *tmp;
+
+      tmp = g_variant_print (v, TRUE);
+      DEBUG ("Unsupported type for param '%s': %s'", param, tmp);
+      g_free (tmp);
     }
 
+  g_variant_unref (v);
   return ret;
 }
 
@@ -1028,35 +1015,33 @@ gint64
 empathy_account_settings_get_int64 (EmpathyAccountSettings *settings,
     const gchar *param)
 {
-  const GValue *v;
+  GVariant *v;
   gint64 ret = 0;
 
-  v = empathy_account_settings_get (settings, param);
+  v = empathy_account_settings_dup (settings, param);
   if (v == NULL)
     return 0;
 
-  switch G_VALUE_TYPE (v)
+  if (g_variant_is_of_type (v, G_VARIANT_TYPE_BYTE))
+    ret = g_variant_get_byte (v);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_INT32))
+    ret = g_variant_get_int32 (v);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_UINT32))
+    ret = g_variant_get_uint32 (v);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_INT64))
+    ret = g_variant_get_int64 (v);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_UINT64))
+    ret = CLAMP (g_variant_get_uint64 (v), (guint64) G_MININT64, G_MAXINT64);
+  else
     {
-      case G_TYPE_UCHAR:
-        ret = g_value_get_uchar (v);
-        break;
-      case G_TYPE_INT:
-        ret = g_value_get_int (v);
-        break;
-      case G_TYPE_UINT:
-        ret = g_value_get_uint (v);
-        break;
-      case G_TYPE_INT64:
-        ret = g_value_get_int64 (v);
-        break;
-      case G_TYPE_UINT64:
-        ret = CLAMP (g_value_get_uint64 (v), (guint64) G_MININT64, G_MAXINT64);
-        break;
-      default:
-        ret = 0;
-        break;
+      gchar *tmp;
+
+      tmp = g_variant_print (v, TRUE);
+      DEBUG ("Unsupported type for param '%s': %s'", param, tmp);
+      g_free (tmp);
     }
 
+  g_variant_unref (v);
   return ret;
 }
 
@@ -1064,35 +1049,33 @@ guint32
 empathy_account_settings_get_uint32 (EmpathyAccountSettings *settings,
     const gchar *param)
 {
-  const GValue *v;
-  guint32 ret;
+  GVariant *v;
+  guint32 ret = 0;
 
-  v = empathy_account_settings_get (settings, param);
+  v = empathy_account_settings_dup (settings, param);
   if (v == NULL)
     return 0;
 
-  switch G_VALUE_TYPE (v)
+  if (g_variant_is_of_type (v, G_VARIANT_TYPE_BYTE))
+    ret = g_variant_get_byte (v);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_INT32))
+    ret = MAX (0, g_variant_get_int32 (v));
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_UINT32))
+    ret = g_variant_get_uint32 (v);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_INT64))
+    ret = CLAMP (g_variant_get_int64 (v), 0, G_MAXUINT32);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_UINT64))
+    ret = MIN (g_variant_get_uint64 (v), G_MAXUINT32);
+  else
     {
-      case G_TYPE_UCHAR:
-        ret = g_value_get_uchar (v);
-        break;
-      case G_TYPE_INT:
-        ret = MAX (0, g_value_get_int (v));
-        break;
-      case G_TYPE_UINT:
-        ret = g_value_get_uint (v);
-        break;
-      case G_TYPE_INT64:
-        ret = CLAMP (g_value_get_int64 (v), 0, G_MAXUINT32);
-        break;
-      case G_TYPE_UINT64:
-        ret = MIN (g_value_get_uint64 (v), G_MAXUINT32);
-        break;
-      default:
-        ret = 0;
-        break;
+      gchar *tmp;
+
+      tmp = g_variant_print (v, TRUE);
+      DEBUG ("Unsupported type for param '%s': %s'", param, tmp);
+      g_free (tmp);
     }
 
+  g_variant_unref (v);
   return ret;
 }
 
@@ -1100,36 +1083,34 @@ guint64
 empathy_account_settings_get_uint64 (EmpathyAccountSettings *settings,
     const gchar *param)
 {
-  const GValue *v;
+  GVariant *v;
   guint64 ret = 0;
 
-  v = empathy_account_settings_get (settings, param);
-
-  if (v == NULL || !G_VALUE_HOLDS_INT (v))
+  v = empathy_account_settings_dup (settings, param);
+  if (v == NULL)
     return 0;
 
-  switch G_VALUE_TYPE (v)
+  if (g_variant_is_of_type (v, G_VARIANT_TYPE_BYTE))
+    ret = g_variant_get_byte (v);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_INT32))
+    ret = MAX (0, g_variant_get_int32 (v));
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_UINT32))
+    ret = g_variant_get_uint32 (v);
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_INT64))
+    ret = MAX (0, g_variant_get_int64 (v));
+  else if (g_variant_is_of_type (v, G_VARIANT_TYPE_UINT64))
+    ret = g_variant_get_uint64 (v);
+  else
     {
-      case G_TYPE_UCHAR:
-        ret = g_value_get_uchar (v);
-        break;
-      case G_TYPE_INT:
-        ret = MAX (0, g_value_get_int (v));
-        break;
-      case G_TYPE_UINT:
-        ret = g_value_get_uint (v);
-        break;
-      case G_TYPE_INT64:
-        ret = MAX (0, g_value_get_int64 (v));
-        break;
-      case G_TYPE_UINT64:
-        ret = g_value_get_uint64 (v);
-        break;
-      default:
-        ret = 0;
-        break;
+      gchar *tmp;
+
+      tmp = g_variant_print (v, TRUE);
+      DEBUG ("Unsupported type for param '%s': %s'", param, tmp);
+      g_free (tmp);
     }
 
+
+  g_variant_unref (v);
   return ret;
 }
 
@@ -1137,125 +1118,45 @@ gboolean
 empathy_account_settings_get_boolean (EmpathyAccountSettings *settings,
     const gchar *param)
 {
-  const GValue *v;
+  GVariant *v;
+  gboolean result = FALSE;
 
-  v = empathy_account_settings_get (settings, param);
+  v = empathy_account_settings_dup (settings, param);
+  if (v == NULL)
+    return result;
 
-  if (v == NULL || !G_VALUE_HOLDS_BOOLEAN (v))
-    return FALSE;
+  if (g_variant_is_of_type (v, G_VARIANT_TYPE_BOOLEAN))
+    result = g_variant_get_boolean (v);
 
-  return g_value_get_boolean (v);
+  return result;
 }
 
 void
-empathy_account_settings_set_string (EmpathyAccountSettings *settings,
+empathy_account_settings_set (EmpathyAccountSettings *settings,
     const gchar *param,
-    const gchar *value)
+    GVariant *v)
 {
   EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
 
   g_return_if_fail (param != NULL);
-  g_return_if_fail (value != NULL);
+  g_return_if_fail (v != NULL);
 
-  if (!tp_strdiff (param, "password") && priv->supports_sasl)
+  if (!tp_strdiff (param, "password") && priv->supports_sasl &&
+      g_variant_is_of_type (v, G_VARIANT_TYPE_STRING))
     {
       g_free (priv->password);
-      priv->password = g_strdup (value);
+      priv->password = g_variant_dup_string (v, NULL);
       priv->password_changed = TRUE;
     }
   else
     {
-      tp_asv_set_string (priv->parameters, g_strdup (param), value);
+      g_hash_table_insert (priv->parameters, g_strdup (param),
+          g_variant_ref_sink (v));
     }
 
   account_settings_remove_from_unset (settings, param);
 }
 
-void
-empathy_account_settings_set_strv (EmpathyAccountSettings *settings,
-    const gchar *param,
-    gchar **value)
-{
-  EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-
-  g_return_if_fail (param != NULL);
-  g_return_if_fail (value != NULL);
-
-  tp_asv_set_strv (priv->parameters, g_strdup (param), value);
-
-  account_settings_remove_from_unset (settings, param);
-}
-
-void
-empathy_account_settings_set_int32 (EmpathyAccountSettings *settings,
-    const gchar *param,
-    gint32 value)
-{
-  EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-
-  g_return_if_fail (param != NULL);
-
-  tp_asv_set_int32 (priv->parameters, g_strdup (param), value);
-
-  account_settings_remove_from_unset (settings, param);
-}
-
-void
-empathy_account_settings_set_int64 (EmpathyAccountSettings *settings,
-    const gchar *param,
-    gint64 value)
-{
-  EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-
-  g_return_if_fail (param != NULL);
-
-  tp_asv_set_int64 (priv->parameters, g_strdup (param), value);
-
-  account_settings_remove_from_unset (settings, param);
-}
-
-void
-empathy_account_settings_set_uint32 (EmpathyAccountSettings *settings,
-    const gchar *param,
-    guint32 value)
-{
-  EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-
-  g_return_if_fail (param != NULL);
-
-  tp_asv_set_uint32 (priv->parameters, g_strdup (param), value);
-
-  account_settings_remove_from_unset (settings, param);
-}
-
-void
-empathy_account_settings_set_uint64 (EmpathyAccountSettings *settings,
-    const gchar *param,
-    guint64 value)
-{
-  EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-
-  g_return_if_fail (param != NULL);
-
-  tp_asv_set_uint64 (priv->parameters, g_strdup (param), value);
-
-  account_settings_remove_from_unset (settings, param);
-}
-
-void
-empathy_account_settings_set_boolean (EmpathyAccountSettings *settings,
-    const gchar *param,
-    gboolean value)
-{
-  EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-
-  g_return_if_fail (param != NULL);
-
-  tp_asv_set_boolean (priv->parameters, g_strdup (param), value);
-
-  account_settings_remove_from_unset (settings, param);
-}
-
 static void
 account_settings_display_name_set_cb (GObject *src,
     GAsyncResult *res,
@@ -1500,7 +1401,7 @@ empathy_account_settings_account_updated (GObject *source,
   GError *error = NULL;
   GStrv reconnect_required = NULL;
 
-  if (!tp_account_update_parameters_finish (TP_ACCOUNT (source),
+  if (!tp_account_update_parameters_vardict_finish (TP_ACCOUNT (source),
           result, &reconnect_required, &error))
     {
       g_simple_async_result_set_from_error (priv->apply_result, error);
@@ -1553,21 +1454,18 @@ empathy_account_settings_created_cb (GObject *source,
 {
   EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (user_data);
   EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-  TpAccount *account;
   GError *error = NULL;
   GSimpleAsyncResult *r;
 
-  account = tp_account_manager_create_account_finish (
-    TP_ACCOUNT_MANAGER (source), result, &error);
+  priv->account = tp_account_request_create_account_finish (
+      TP_ACCOUNT_REQUEST (source), result, &error);
 
-  if (account == NULL)
+  if (priv->account == NULL)
     {
       g_simple_async_result_set_from_error (priv->apply_result, error);
     }
   else
     {
-      priv->account = g_object_ref (account);
-
       if (priv->supports_sasl && priv->password != NULL)
         {
           /* Save the password before connecting */
@@ -1592,18 +1490,20 @@ empathy_account_settings_created_cb (GObject *source,
   g_object_unref (r);
 }
 
-
 static void
-empathy_account_settings_do_create_account (EmpathyAccountSettings *settings)
+empathy_account_settings_do_create_account (EmpathyAccountSettings *self)
 {
-  EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-  GHashTable *properties;
+  EmpathyAccountSettingsPriv *priv = GET_PRIV (self);
+  TpAccountRequest *account_req;
   TpConnectionPresenceType type;
   gchar *status;
   gchar *message;
   EmpathyPresenceManager *presence_mgr;
+  GHashTableIter iter;
+  gpointer k, v;
 
-  properties = tp_asv_new (NULL, NULL);
+  account_req = tp_account_request_new (priv->account_manager, priv->cm_name,
+      priv->protocol, "New Account");
 
   presence_mgr = empathy_presence_manager_dup_singleton ();
   type = empathy_presence_manager_get_requested_presence (presence_mgr, &status,
@@ -1612,65 +1512,54 @@ empathy_account_settings_do_create_account (EmpathyAccountSettings *settings)
 
   if (type != TP_CONNECTION_PRESENCE_TYPE_UNSET)
     {
-      /* Create the account with the requested presence the same as the current
-        * global requested presence, but don't enable it */
-      GValueArray *presence;
-      GValue vtype = { 0, };
-      GValue vstatus = { 0, };
-      GValue vmessage = { 0, };
-
-      presence = g_value_array_new (3);
-
-      g_value_init (&vtype, G_TYPE_UINT);
-      g_value_set_uint (&vtype, type);
-      g_value_array_append (presence, &vtype);
-
-      g_value_init (&vstatus, G_TYPE_STRING);
-      g_value_take_string (&vstatus, status);
-      g_value_array_append (presence, &vstatus);
-
-      g_value_init (&vmessage, G_TYPE_STRING);
-      g_value_take_string (&vmessage, message);
-      g_value_array_append (presence, &vmessage);
-
-      tp_asv_take_boxed (properties, TP_IFACE_ACCOUNT ".RequestedPresence",
-        TP_STRUCT_TYPE_SIMPLE_PRESENCE, presence);
+      tp_account_request_set_requested_presence (account_req, type,
+          status, message);
     }
 
-  tp_asv_set_string (properties, TP_IFACE_ACCOUNT ".Icon",
-      priv->icon_name);
+  tp_account_request_set_icon_name (account_req, priv->icon_name);
+
+  tp_account_request_set_display_name (account_req, priv->display_name);
 
   if (priv->service != NULL)
-    tp_asv_set_string (properties, TP_PROP_ACCOUNT_SERVICE, priv->service);
+    tp_account_request_set_service (account_req, priv->service);
 
-  tp_account_manager_create_account_async (priv->account_manager,
-    priv->cm_name, priv->protocol, priv->display_name,
-    priv->parameters, properties,
-    empathy_account_settings_created_cb,
-    settings);
-
-  g_hash_table_unref (properties);
-}
-
-static void
-empathy_account_settings_manager_ready_cb (GObject *source_object,
-    GAsyncResult *result,
-    gpointer user_data)
-{
-  EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (user_data);
-  EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
-  TpAccountManager *account_manager = TP_ACCOUNT_MANAGER (source_object);
-  GError *error = NULL;
-
-  if (!tp_proxy_prepare_finish (account_manager, result, &error))
+  g_hash_table_iter_init (&iter, priv->parameters);
+  while (g_hash_table_iter_next (&iter, &k, &v))
     {
-      DEBUG ("Failed to prepare account manager: %s", error->message);
-      g_error_free (error);
-      return;
+      const gchar *key = k;
+      GVariant *value = v;
+
+      tp_account_request_set_parameter (account_req, key, value);
     }
 
-  g_assert (priv->apply_result != NULL && priv->account == NULL);
-  empathy_account_settings_do_create_account (settings);
+  tp_account_request_create_account_async (account_req,
+      empathy_account_settings_created_cb, self);
+}
+
+static GVariant *
+build_parameters_variant (EmpathyAccountSettings *self)
+{
+  EmpathyAccountSettingsPriv *priv = GET_PRIV (self);
+  GVariantBuilder *builder;
+  GHashTableIter iter;
+  gpointer k, v;
+
+  builder = g_variant_builder_new (G_VARIANT_TYPE_VARDICT);
+
+  g_hash_table_iter_init (&iter, priv->parameters);
+  while (g_hash_table_iter_next (&iter, &k, &v))
+    {
+      const gchar *key = k;
+      GVariant *value = v;
+      GVariant *entry;
+
+      entry = g_variant_new_dict_entry (g_variant_new_string (key),
+          g_variant_new_variant (value));
+
+      g_variant_builder_add_value (builder, entry);
+    }
+
+  return g_variant_builder_end (builder);
 }
 
 void
@@ -1697,13 +1586,15 @@ empathy_account_settings_apply_async (EmpathyAccountSettings *settings,
 
   if (priv->account == NULL)
     {
-      tp_proxy_prepare_async (priv->account_manager, NULL,
-          empathy_account_settings_manager_ready_cb, settings);
+      g_assert (priv->apply_result != NULL && priv->account == NULL);
+
+      empathy_account_settings_do_create_account (settings);
     }
   else
     {
-      tp_account_update_parameters_async (priv->account,
-          priv->parameters, (const gchar **)priv->unset_parameters->data,
+      tp_account_update_parameters_vardict_async (priv->account,
+          build_parameters_variant (settings),
+          (const gchar **) priv->unset_parameters->data,
           empathy_account_settings_account_updated, settings);
     }
 }
@@ -1777,7 +1668,6 @@ empathy_account_settings_parameter_is_valid (
 {
   EmpathyAccountSettingsPriv *priv;
   const GRegex *regex;
-  const gchar *value;
 
   g_return_val_if_fail (EMPATHY_IS_ACCOUNT_SETTINGS (settings), FALSE);
 
@@ -1786,7 +1676,7 @@ empathy_account_settings_parameter_is_valid (
   if (g_list_find_custom (priv->required_params, param, (GCompareFunc) strcmp))
     {
       /* first, look if it's set in our own parameters */
-      if (tp_asv_lookup (priv->parameters, param))
+      if (g_hash_table_lookup (priv->parameters, param) != NULL)
         goto test_regex;
 
       /* if we did not unset the parameter, look if it's in the account */
@@ -1808,9 +1698,17 @@ test_regex:
   regex = g_hash_table_lookup (priv->param_regexps, param);
   if (regex)
     {
-      value = empathy_account_settings_get_string (settings, param);
-      if (value != NULL && !g_regex_match (regex, value, 0, NULL))
+      gchar *value;
+      gboolean match;
+
+      value = empathy_account_settings_dup_string (settings, param);
+      if (value == NULL)
         return FALSE;
+
+      match = g_regex_match (regex, value, 0, NULL);
+
+      g_free (value);
+      return match;
     }
 
   return TRUE;
@@ -1844,12 +1742,12 @@ empathy_account_settings_is_valid (EmpathyAccountSettings *settings)
   return TRUE;
 }
 
-const TpConnectionManagerProtocol *
+TpProtocol *
 empathy_account_settings_get_tp_protocol (EmpathyAccountSettings *self)
 {
   EmpathyAccountSettingsPriv *priv = GET_PRIV (self);
 
-  return tp_connection_manager_get_protocol (priv->manager, priv->protocol);
+  return priv->protocol_obj;
 }
 
 gboolean
