@@ -94,10 +94,9 @@ gboolean empathy_tp_chat_can_add_contact (EmpathyTpChat *self);
 
 void empathy_tp_chat_leave (EmpathyTpChat *chat,
     const gchar *message);
-void empathy_tp_chat_join (EmpathyTpChat *chat);
 
 gboolean empathy_tp_chat_is_invited (EmpathyTpChat *chat,
-    TpHandle *inviter);
+    TpContact **inviter);
 TpChannelChatState empathy_tp_chat_get_chat_state (EmpathyTpChat *chat,
     EmpathyContact *contact);
 
