@@ -439,36 +439,6 @@ empathy_file_lookup (const gchar *filename, const gchar *subdir)
   return path;
 }
 
-guint
-empathy_proxy_hash (gconstpointer key)
-{
-  TpProxy *proxy = TP_PROXY (key);
-  TpProxyClass *proxy_class = TP_PROXY_GET_CLASS (key);
-
-  g_return_val_if_fail (TP_IS_PROXY (proxy), 0);
-  g_return_val_if_fail (proxy_class->must_have_unique_name, 0);
-
-  return g_str_hash (proxy->object_path) ^ g_str_hash (proxy->bus_name);
-}
-
-gboolean
-empathy_proxy_equal (gconstpointer a,
-    gconstpointer b)
-{
-  TpProxy *proxy_a = TP_PROXY (a);
-  TpProxy *proxy_b = TP_PROXY (b);
-  TpProxyClass *proxy_a_class = TP_PROXY_GET_CLASS (a);
-  TpProxyClass *proxy_b_class = TP_PROXY_GET_CLASS (b);
-
-  g_return_val_if_fail (TP_IS_PROXY (proxy_a), FALSE);
-  g_return_val_if_fail (TP_IS_PROXY (proxy_b), FALSE);
-  g_return_val_if_fail (proxy_a_class->must_have_unique_name, 0);
-  g_return_val_if_fail (proxy_b_class->must_have_unique_name, 0);
-
-  return g_str_equal (proxy_a->object_path, proxy_b->object_path) &&
-         g_str_equal (proxy_a->bus_name, proxy_b->bus_name);
-}
-
 gboolean
 empathy_check_available_state (void)
 {
@@ -1273,3 +1243,32 @@ while_finish:
 
   return types;
 }
+
+GVariant *
+empathy_asv_to_vardict (const GHashTable *asv)
+{
+  return empathy_boxed_to_variant (TP_HASH_TYPE_STRING_VARIANT_MAP, "a{sv}",
+      (gpointer) asv);
+}
+
+GVariant *
+empathy_boxed_to_variant (GType gtype,
+    const gchar *variant_type,
+    gpointer boxed)
+{
+  GValue v = G_VALUE_INIT;
+  GVariant *ret;
+
+  g_return_val_if_fail (boxed != NULL, NULL);
+
+  g_value_init (&v, gtype);
+  g_value_set_boxed (&v, boxed);
+
+  ret = dbus_g_value_build_g_variant (&v);
+  g_return_val_if_fail (!tp_strdiff (g_variant_get_type_string (ret),
+        variant_type), NULL);
+
+  g_value_unset (&v);
+
+  return g_variant_ref_sink (ret);
+}
