@@ -44,7 +44,6 @@
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-request-util.h>
 #include <libempathy/empathy-individual-manager.h>
 
@@ -61,7 +60,6 @@
 #include "empathy-chat-window.h"
 #include "empathy-about-dialog.h"
 #include "empathy-invite-participant-dialog.h"
-#include "gedit-close-button.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
@@ -413,6 +411,35 @@ chat_tab_style_updated_cb (GtkWidget *hbox,
 	gtk_widget_set_size_request (button, w, h);
 }
 
+static GtkWidget *
+create_close_button (void)
+{
+	GtkWidget *button, *image;
+	GtkStyleContext *context;
+
+	button = gtk_button_new ();
+
+	context = gtk_widget_get_style_context (button);
+	gtk_style_context_add_class (context, "empathy-tab-close-button");
+
+	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+	gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
+
+	/* We don't want focus/keynav for the button to avoid clutter, and
+	 * Ctrl-W works anyway.
+	 */
+	gtk_widget_set_can_focus (button, FALSE);
+	gtk_widget_set_can_default (button, FALSE);
+
+	image = gtk_image_new_from_icon_name ("window-close-symbolic",
+	                                      GTK_ICON_SIZE_MENU);
+	gtk_widget_show (image);
+
+	gtk_container_add (GTK_CONTAINER (button), image);
+
+	return button;
+}
+
 static GtkWidget *
 chat_window_create_label (EmpathyChatWindow *window,
 			  EmpathyChat       *chat,
@@ -480,15 +507,9 @@ chat_window_create_label (EmpathyChatWindow *window,
 			"chat-window-tab-sending-spinner",
 			sending_spinner);
 
-		close_button = gedit_close_button_new ();
+		close_button = create_close_button ();
 		g_object_set_data (G_OBJECT (chat), "chat-window-tab-close-button", close_button);
 
-		/* We don't want focus/keynav for the button to avoid clutter, and
-		 * Ctrl-W works anyway.
-		 */
-		gtk_widget_set_can_focus (close_button, FALSE);
-		gtk_widget_set_can_default (close_button, FALSE);
-
 		gtk_box_pack_end (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
 
 		g_signal_connect (close_button,
@@ -2308,6 +2329,8 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 	g_object_ref (priv->ui_manager);
 	g_object_unref (gui);
 
+	empathy_set_css_provider (GTK_WIDGET (priv->dialog));
+
 	priv->gsettings_chat = g_settings_new (EMPATHY_PREFS_CHAT_SCHEMA);
 	priv->gsettings_notif = g_settings_new (EMPATHY_PREFS_NOTIFICATIONS_SCHEMA);
 	priv->gsettings_ui = g_settings_new (EMPATHY_PREFS_UI_SCHEMA);
