@@ -49,9 +49,8 @@
 #include <libempathy-gtk/empathy-geometry.h>
 #include <libempathy-gtk/empathy-gtk-enum-types.h>
 #include <libempathy-gtk/empathy-individual-dialogs.h>
-#include <libempathy-gtk/empathy-individual-store.h>
 #include <libempathy-gtk/empathy-individual-store-manager.h>
-#include <libempathy-gtk/empathy-individual-view.h>
+#include <libempathy-gtk/empathy-roster-view.h>
 #include <libempathy-gtk/empathy-new-message-dialog.h>
 #include <libempathy-gtk/empathy-new-call-dialog.h>
 #include <libempathy-gtk/empathy-log-window.h>
@@ -101,8 +100,7 @@ enum
 G_DEFINE_TYPE (EmpathyRosterWindow, empathy_roster_window, GTK_TYPE_APPLICATION_WINDOW)
 
 struct _EmpathyRosterWindowPriv {
-  EmpathyIndividualStore *individual_store;
-  EmpathyIndividualView *individual_view;
+  EmpathyRosterView *view;
   TpAccountManager *account_manager;
   EmpathyChatroomManager *chatroom_manager;
   EmpathyEventManager *event_manager;
@@ -111,10 +109,8 @@ struct _EmpathyRosterWindowPriv {
   EmpathyIndividualManager *individual_manager;
   guint flash_timeout_id;
   gboolean flash_on;
-  gboolean empty;
 
   GSettings *gsettings_ui;
-  GSettings *gsettings_contacts;
 
   GtkWidget *preferences;
   GtkWidget *main_vbox;
@@ -128,6 +124,7 @@ struct _EmpathyRosterWindowPriv {
   GtkWidget *no_entry_label;
   GtkWidget *button_account_settings;
   GtkWidget *spinner_loading;
+  GtkWidget *tooltip_widget;
 
   GMenu *menumodel;
   GMenu *rooms_section;
@@ -152,136 +149,6 @@ struct _EmpathyRosterWindowPriv {
   gboolean shell_running;
 };
 
-static void
-roster_window_flash_stop (EmpathyRosterWindow *self)
-{
-  if (self->priv->flash_timeout_id == 0)
-    return;
-
-  DEBUG ("Stop flashing");
-  g_source_remove (self->priv->flash_timeout_id);
-  self->priv->flash_timeout_id = 0;
-  self->priv->flash_on = FALSE;
-}
-
-typedef struct
-{
-  EmpathyEvent *event;
-  gboolean on;
-  EmpathyRosterWindow *self;
-} FlashForeachData;
-
-static gboolean
-roster_window_flash_foreach (GtkTreeModel *model,
-    GtkTreePath *path,
-    GtkTreeIter *iter,
-    gpointer user_data)
-{
-  FlashForeachData *data = (FlashForeachData *) user_data;
-  FolksIndividual *individual;
-  EmpathyContact *contact;
-  const gchar *icon_name;
-  GtkTreePath *parent_path = NULL;
-  GtkTreeIter parent_iter;
-  GdkPixbuf *pixbuf = NULL;
-
-  gtk_tree_model_get (model, iter,
-          EMPATHY_INDIVIDUAL_STORE_COL_INDIVIDUAL, &individual,
-          -1);
-
-  if (individual == NULL)
-    return FALSE;
-
-  contact = empathy_contact_dup_from_folks_individual (individual);
-  if (contact != data->event->contact)
-    goto out;
-
-  if (data->on)
-    {
-      icon_name = data->event->icon_name;
-      pixbuf = empathy_pixbuf_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
-    }
-  else
-    {
-      pixbuf = empathy_individual_store_get_individual_status_icon (
-              data->self->priv->individual_store,
-              individual);
-      if (pixbuf != NULL)
-        g_object_ref (pixbuf);
-    }
-
-  gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-      EMPATHY_INDIVIDUAL_STORE_COL_ICON_STATUS, pixbuf,
-      -1);
-
-  /* To make sure the parent is shown correctly, we emit
-   * the row-changed signal on the parent so it prompts
-   * it to be refreshed by the filter func.
-   */
-  if (gtk_tree_model_iter_parent (model, &parent_iter, iter))
-    {
-      parent_path = gtk_tree_model_get_path (model, &parent_iter);
-    }
-
-  if (parent_path != NULL)
-    {
-      gtk_tree_model_row_changed (model, parent_path, &parent_iter);
-      gtk_tree_path_free (parent_path);
-    }
-
-out:
-  g_object_unref (individual);
-  tp_clear_object (&contact);
-  tp_clear_object (&pixbuf);
-
-  return FALSE;
-}
-
-static gboolean
-roster_window_flash_cb (EmpathyRosterWindow *self)
-{
-  GtkTreeModel *model;
-  GSList *events, *l;
-  gboolean found_event = FALSE;
-  FlashForeachData  data;
-
-  self->priv->flash_on = !self->priv->flash_on;
-  data.on = self->priv->flash_on;
-  model = GTK_TREE_MODEL (self->priv->individual_store);
-
-  events = empathy_event_manager_get_events (self->priv->event_manager);
-  for (l = events; l; l = l->next)
-    {
-      data.event = l->data;
-      data.self = self;
-      if (!data.event->contact || !data.event->must_ack)
-        continue;
-
-      found_event = TRUE;
-      gtk_tree_model_foreach (model,
-          roster_window_flash_foreach,
-          &data);
-    }
-
-  if (!found_event)
-    roster_window_flash_stop (self);
-
-  return TRUE;
-}
-
-static void
-roster_window_flash_start (EmpathyRosterWindow *self)
-{
-  if (self->priv->flash_timeout_id != 0)
-    return;
-
-  DEBUG ("Start flashing");
-  self->priv->flash_timeout_id = g_timeout_add (FLASH_TIMEOUT,
-      (GSourceFunc) roster_window_flash_cb, self);
-
-  roster_window_flash_cb (self);
-}
-
 static void
 roster_window_remove_auth (EmpathyRosterWindow *self,
     EmpathyEvent *event)
@@ -406,81 +273,16 @@ roster_window_auth_display (EmpathyRosterWindow *self,
   g_hash_table_insert (self->priv->auths, event, info_bar);
 }
 
-static void
-modify_event_count (GtkTreeModel *model,
-    GtkTreeIter *iter,
-    EmpathyEvent *event,
-    gboolean increase)
+static FolksIndividual *
+ensure_individual_for_event (EmpathyEvent *event)
 {
-  FolksIndividual *individual;
-  EmpathyContact *contact;
-  guint count;
+  TpContact *contact;
 
-  gtk_tree_model_get (model, iter,
-      EMPATHY_INDIVIDUAL_STORE_COL_INDIVIDUAL, &individual,
-      EMPATHY_INDIVIDUAL_STORE_COL_EVENT_COUNT, &count,
-      -1);
+  contact = empathy_contact_get_tp_contact (event->contact);
+  if (contact == NULL)
+    return NULL;
 
-  if (individual == NULL)
-    return;
-
-  increase ? count++ : count--;
-
-  contact = empathy_contact_dup_from_folks_individual (individual);
-  if (contact == event->contact)
-    gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-        EMPATHY_INDIVIDUAL_STORE_COL_EVENT_COUNT, count, -1);
-
-  tp_clear_object (&contact);
-  g_object_unref (individual);
-}
-
-static gboolean
-increase_event_count_foreach (GtkTreeModel *model,
-    GtkTreePath *path,
-    GtkTreeIter *iter,
-    gpointer user_data)
-{
-  EmpathyEvent *event = user_data;
-
-  modify_event_count (model, iter, event, TRUE);
-
-  return FALSE;
-}
-
-static void
-increase_event_count (EmpathyRosterWindow *self,
-    EmpathyEvent *event)
-{
-  GtkTreeModel *model;
-
-  model = GTK_TREE_MODEL (self->priv->individual_store);
-
-  gtk_tree_model_foreach (model, increase_event_count_foreach, event);
-}
-
-static gboolean
-decrease_event_count_foreach (GtkTreeModel *model,
-    GtkTreePath *path,
-    GtkTreeIter *iter,
-    gpointer user_data)
-{
-  EmpathyEvent *event = user_data;
-
-  modify_event_count (model, iter, event, FALSE);
-
-  return FALSE;
-}
-
-static void
-decrease_event_count (EmpathyRosterWindow *self,
-    EmpathyEvent *event)
-{
-  GtkTreeModel *model;
-
-  model = GTK_TREE_MODEL (self->priv->individual_store);
-
-  gtk_tree_model_foreach (model, decrease_event_count_foreach, event);
+  return empathy_ensure_individual_from_tp_contact (contact);
 }
 
 static void
@@ -490,9 +292,16 @@ roster_window_event_added_cb (EmpathyEventManager *manager,
 {
   if (event->contact)
     {
-      increase_event_count (self, event);
+      FolksIndividual *individual;
 
-      roster_window_flash_start (self);
+      individual = ensure_individual_for_event (event);
+      if (individual == NULL)
+        return;
+
+      event->roster_view_id = empathy_roster_view_add_event (self->priv->view,
+          individual, event->icon_name, event);
+
+      g_object_unref (individual);
     }
   else if (event->type == EMPATHY_EVENT_TYPE_AUTH)
     {
@@ -505,26 +314,13 @@ roster_window_event_removed_cb (EmpathyEventManager *manager,
     EmpathyEvent *event,
     EmpathyRosterWindow *self)
 {
-  FlashForeachData data;
-
   if (event->type == EMPATHY_EVENT_TYPE_AUTH)
     {
       roster_window_remove_auth (self, event);
       return;
     }
 
-  if (!event->contact)
-    return;
-
-  decrease_event_count (self, event);
-
-  data.on = FALSE;
-  data.event = event;
-  data.self = self;
-
-  gtk_tree_model_foreach (GTK_TREE_MODEL (self->priv->individual_store),
-      roster_window_flash_foreach,
-      &data);
+  empathy_roster_view_remove_event (self->priv->view, event->roster_view_id);
 }
 
 static gboolean
@@ -545,51 +341,33 @@ roster_window_load_events_idle_cb (gpointer user_data)
 }
 
 static void
-roster_window_row_activated_cb (EmpathyIndividualView *view,
-    GtkTreePath *path,
-    GtkTreeViewColumn *col,
-    EmpathyRosterWindow *self)
+individual_activated_cb (EmpathyRosterView *self,
+    FolksIndividual *individual,
+    gpointer user_data)
 {
-  EmpathyContact *contact = NULL;
-  FolksIndividual *individual;
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-  GSList *events, *l;
+  EmpathyContact *contact;
 
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (self->priv->individual_view));
-  gtk_tree_model_get_iter (model, &iter, path);
-
-  gtk_tree_model_get (model, &iter, EMPATHY_INDIVIDUAL_STORE_COL_INDIVIDUAL,
-      &individual, -1);
-
-  if (individual != NULL)
-    contact = empathy_contact_dup_from_folks_individual (individual);
+  contact = empathy_contact_dup_best_for_action (individual,
+      EMPATHY_ACTION_CHAT);
 
   if (contact == NULL)
-    goto OUT;
+    return;
 
-  /* If the contact has an event activate it, otherwise the
-   * default handler of row-activated will be called. */
-  events = empathy_event_manager_get_events (self->priv->event_manager);
-  for (l = events; l; l = l->next)
-    {
-      EmpathyEvent *event = l->data;
+  DEBUG ("Starting a chat");
 
-      if (event->contact == contact)
-        {
-          DEBUG ("Activate event");
-          empathy_event_activate (event);
-
-          /* We don't want the default handler of this signal
-           * (e.g. open a chat) */
-          g_signal_stop_emission_by_name (view, "row-activated");
-          break;
-        }
-    }
+  empathy_chat_with_contact (contact, gtk_get_current_event_time ());
 
   g_object_unref (contact);
-OUT:
-  tp_clear_object (&individual);
+}
+
+static void
+event_activated_cb (EmpathyRosterView *self,
+    FolksIndividual *individual,
+    gpointer user_data)
+{
+  EmpathyEvent *event = user_data;
+
+  empathy_event_activate (event);
 }
 
 static void
@@ -639,24 +417,6 @@ display_page_no_account (EmpathyRosterWindow *self)
       _("You need to setup an account to see contacts here."), TRUE, FALSE);
 }
 
-static void
-roster_window_row_deleted_cb (GtkTreeModel *model,
-    GtkTreePath *path,
-    EmpathyRosterWindow *self)
-{
-  GtkTreeIter help_iter;
-
-  if (!gtk_tree_model_get_iter_first (model, &help_iter))
-    {
-      self->priv->empty = TRUE;
-
-      if (empathy_individual_view_is_searching (self->priv->individual_view))
-        {
-          display_page_message (self, _("No match found"), FALSE, FALSE);
-        }
-    }
-}
-
 static void
 display_page_contact_list (EmpathyRosterWindow *self)
 {
@@ -669,27 +429,6 @@ display_page_contact_list (EmpathyRosterWindow *self)
       PAGE_CONTACT_LIST);
 }
 
-static void
-roster_window_row_inserted_cb (GtkTreeModel      *model,
-    GtkTreePath *path,
-    GtkTreeIter *iter,
-    EmpathyRosterWindow *self)
-{
-  if (self->priv->empty)
-    {
-      self->priv->empty = FALSE;
-
-      display_page_contact_list (self);
-      gtk_widget_grab_focus (GTK_WIDGET (self->priv->individual_view));
-
-      /* The store is being filled, it will be done after an idle cb.
-       * So we can then get events. If we do that too soon, event's
-       * contact is not yet in the store and it won't get marked as
-       * having events. */
-      g_idle_add (roster_window_load_events_idle_cb, self);
-    }
-}
-
 static void
 roster_window_remove_error (EmpathyRosterWindow *self,
     TpAccount *account)
@@ -1240,7 +979,6 @@ empathy_roster_window_finalize (GObject *window)
   g_list_free (self->priv->actions_connected);
 
   g_object_unref (self->priv->account_manager);
-  g_object_unref (self->priv->individual_store);
   g_object_unref (self->priv->sound_mgr);
   g_hash_table_unref (self->priv->errors);
   g_hash_table_unref (self->priv->auths);
@@ -1252,22 +990,18 @@ empathy_roster_window_finalize (GObject *window)
 
   g_hash_table_unref (self->priv->status_changed_handlers);
 
-  g_signal_handlers_disconnect_by_func (self->priv->event_manager,
-      roster_window_event_added_cb, self);
-  g_signal_handlers_disconnect_by_func (self->priv->event_manager,
-      roster_window_event_removed_cb, self);
-
   g_object_unref (self->priv->call_observer);
   g_object_unref (self->priv->event_manager);
   g_object_unref (self->priv->chatroom_manager);
 
   g_object_unref (self->priv->gsettings_ui);
-  g_object_unref (self->priv->gsettings_contacts);
   g_object_unref (self->priv->individual_manager);
 
   g_object_unref (self->priv->menumodel);
   g_object_unref (self->priv->rooms_section);
 
+  g_clear_object (&self->priv->tooltip_widget);
+
   G_OBJECT_CLASS (empathy_roster_window_parent_class)->finalize (window);
 }
 
@@ -1356,16 +1090,6 @@ roster_window_view_show_ft_manager (GSimpleAction *action,
   empathy_ft_manager_show ();
 }
 
-static void
-roster_window_edit_search_contacts_cb (GSimpleAction *action,
-    GVariant *parameter,
-    gpointer user_data)
-{
-  EmpathyRosterWindow *self = user_data;
-
-  empathy_individual_view_start_search (self->priv->individual_view);
-}
-
 static void
 join_chatroom (EmpathyChatroom *chatroom,
     gint64 timestamp)
@@ -1709,14 +1433,6 @@ roster_window_help_about_cb (GSimpleAction *action,
   empathy_about_dialog_new (GTK_WINDOW (self));
 }
 
-static void
-roster_window_help_debug_cb (GSimpleAction *action,
-    GVariant *parameter,
-    gpointer user_data)
-{
-  empathy_launch_program (BIN_DIR, "empathy-debugger", NULL);
-}
-
 static void
 roster_window_help_contents_cb (GSimpleAction *action,
     GVariant *parameter,
@@ -1903,7 +1619,6 @@ roster_window_connection_items_setup (EmpathyRosterWindow *self)
       "chat_search_contacts",
       "chat_add_contact",
       "edit_blocked_contacts",
-      "edit_search_contacts"
   };
 
   for (i = 0; i < G_N_ELEMENTS (actions_connected); i++)
@@ -2010,7 +1725,6 @@ static GActionEntry menubar_entries[] = {
   { "chat_quit", roster_window_chat_quit_cb, NULL, NULL, NULL },
 
   { "edit_accounts", roster_window_edit_accounts_cb, NULL, NULL, NULL },
-  { "edit_search_contacts", roster_window_edit_search_contacts_cb, NULL, NULL, NULL },
   { "edit_blocked_contacts", roster_window_edit_blocked_contacts_cb, NULL, NULL, NULL },
   { "edit_preferences", roster_window_edit_preferences_cb, NULL, NULL, NULL },
 
@@ -2022,7 +1736,6 @@ static GActionEntry menubar_entries[] = {
   { "room_manage_favorites", roster_window_room_manage_favorites_cb, NULL, NULL, NULL },
 
   { "help_contents", roster_window_help_contents_cb, NULL, NULL, NULL },
-  { "help_debug", roster_window_help_debug_cb, NULL, NULL, NULL },
   { "help_about", roster_window_help_about_cb, NULL, NULL, NULL },
 };
 
@@ -2132,13 +1845,122 @@ roster_window_setup_actions (EmpathyRosterWindow *self)
 #undef ADD_GSETTINGS_ACTION
 }
 
+static void
+menu_deactivate_cb (GtkMenuShell *menushell,
+    gpointer user_data)
+{
+  /* FIXME: we shouldn't have to disconnect the signal (bgo #641327) */
+  g_signal_handlers_disconnect_by_func (menushell,
+      menu_deactivate_cb, user_data);
+
+  gtk_menu_detach (GTK_MENU (menushell));
+}
+
+static void
+popup_individual_menu_cb (EmpathyRosterView *view,
+    FolksIndividual *individual,
+    guint button,
+    guint time,
+    gpointer user_data)
+{
+  GtkWidget *menu;
+  EmpathyIndividualFeatureFlags features = EMPATHY_INDIVIDUAL_FEATURE_CHAT |
+    EMPATHY_INDIVIDUAL_FEATURE_CALL |
+    EMPATHY_INDIVIDUAL_FEATURE_EDIT |
+    EMPATHY_INDIVIDUAL_FEATURE_INFO |
+    EMPATHY_INDIVIDUAL_FEATURE_LOG |
+    EMPATHY_INDIVIDUAL_FEATURE_SMS |
+    EMPATHY_INDIVIDUAL_FEATURE_CALL_PHONE |
+    EMPATHY_INDIVIDUAL_FEATURE_REMOVE;
+
+  menu = empathy_individual_menu_new (individual, features, NULL);
+
+  /* menu is initially unowned but gtk_menu_attach_to_widget() takes its
+   * floating ref. We can either wait for the view to release its ref
+   * when it is destroyed (when leaving Empathy) or explicitly
+   * detach the menu when it's not displayed any more.
+   * We go for the latter as we don't want to keep useless menus in memory
+   * during the whole lifetime of Empathy. */
+  g_signal_connect (menu, "deactivate", G_CALLBACK (menu_deactivate_cb),
+      NULL);
+
+  gtk_menu_attach_to_widget (GTK_MENU (menu), GTK_WIDGET (view), NULL);
+  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, time);
+}
+
+static void
+view_empty_cb (EmpathyRosterView *view,
+    GParamSpec *spec,
+    EmpathyRosterWindow *self)
+{
+  if (empathy_roster_view_is_empty (view))
+    {
+      if (empathy_roster_view_is_searching (self->priv->view))
+        {
+          display_page_message (self, _("No match found"), FALSE, FALSE);
+        }
+    }
+  else
+    {
+      display_page_contact_list (self);
+      gtk_widget_grab_focus (GTK_WIDGET (self->priv->view));
+
+      /* The store is being filled, it will be done after an idle cb.
+       * So we can then get events. If we do that too soon, event's
+       * contact is not yet in the store and it won't get marked as
+       * having events. */
+      g_idle_add (roster_window_load_events_idle_cb, self);
+    }
+}
+
+static void
+tooltip_destroy_cb (GtkWidget *widget,
+    EmpathyRosterWindow *self)
+{
+  g_clear_object (&self->priv->tooltip_widget);
+}
+
+static gboolean
+individual_tooltip_cb (EmpathyRosterView *view,
+    FolksIndividual *individual,
+    gboolean keyboard_mode,
+    GtkTooltip *tooltip,
+    EmpathyRosterWindow *self)
+{
+  if (self->priv->tooltip_widget == NULL)
+    {
+      self->priv->tooltip_widget = empathy_individual_widget_new (individual,
+          EMPATHY_INDIVIDUAL_WIDGET_FOR_TOOLTIP |
+          EMPATHY_INDIVIDUAL_WIDGET_SHOW_LOCATION |
+          EMPATHY_INDIVIDUAL_WIDGET_SHOW_CLIENT_TYPES);
+
+      gtk_container_set_border_width (
+          GTK_CONTAINER (self->priv->tooltip_widget), 8);
+
+      g_object_ref (self->priv->tooltip_widget);
+
+      tp_g_signal_connect_object (self->priv->tooltip_widget, "destroy",
+          G_CALLBACK (tooltip_destroy_cb), self, 0);
+
+      gtk_widget_show (self->priv->tooltip_widget);
+    }
+  else
+    {
+      empathy_individual_widget_set_individual (
+        EMPATHY_INDIVIDUAL_WIDGET (self->priv->tooltip_widget), individual);
+    }
+
+  gtk_tooltip_set_custom (tooltip, self->priv->tooltip_widget);
+
+  return TRUE;
+}
+
 static void
 empathy_roster_window_init (EmpathyRosterWindow *self)
 {
   GtkBuilder *gui;
   GtkWidget *sw;
   gchar *filename;
-  GtkTreeModel *model;
   GtkWidget *search_vbox;
 
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
@@ -2147,7 +1969,6 @@ empathy_roster_window_init (EmpathyRosterWindow *self)
   empathy_set_css_provider (GTK_WIDGET (self));
 
   self->priv->gsettings_ui = g_settings_new (EMPATHY_PREFS_UI_SCHEMA);
-  self->priv->gsettings_contacts = g_settings_new (EMPATHY_PREFS_CONTACTS_SCHEMA);
 
   self->priv->sound_mgr = empathy_sound_manager_dup_singleton ();
 
@@ -2250,59 +2071,37 @@ empathy_roster_window_init (EmpathyRosterWindow *self)
           "contacts-loaded", G_CALLBACK (contacts_loaded_cb), self, 0);
     }
 
-  self->priv->individual_store = EMPATHY_INDIVIDUAL_STORE (
-      empathy_individual_store_manager_new (self->priv->individual_manager));
+  self->priv->view = EMPATHY_ROSTER_VIEW (
+      empathy_roster_view_new (self->priv->individual_manager));
 
-  /* For the moment, we disallow Persona drops onto the roster contact list
-   * (e.g. from things such as the EmpathyPersonaView in the linking dialogue).
-   * No code is hooked up to do anything on a Persona drop, so allowing them
-   * would achieve nothing except confusion. */
-  self->priv->individual_view = empathy_individual_view_new (
-      self->priv->individual_store,
-      /* EmpathyIndividualViewFeatureFlags */
-      EMPATHY_INDIVIDUAL_VIEW_FEATURE_GROUPS_SAVE |
-      EMPATHY_INDIVIDUAL_VIEW_FEATURE_GROUPS_RENAME |
-      EMPATHY_INDIVIDUAL_VIEW_FEATURE_GROUPS_REMOVE |
-      EMPATHY_INDIVIDUAL_VIEW_FEATURE_GROUPS_CHANGE |
-      EMPATHY_INDIVIDUAL_VIEW_FEATURE_INDIVIDUAL_REMOVE |
-      EMPATHY_INDIVIDUAL_VIEW_FEATURE_INDIVIDUAL_DROP |
-      EMPATHY_INDIVIDUAL_VIEW_FEATURE_INDIVIDUAL_DRAG |
-      EMPATHY_INDIVIDUAL_VIEW_FEATURE_INDIVIDUAL_TOOLTIP |
-      EMPATHY_INDIVIDUAL_VIEW_FEATURE_INDIVIDUAL_CALL |
-      EMPATHY_INDIVIDUAL_VIEW_FEATURE_FILE_DROP,
-      /* EmpathyIndividualFeatureFlags  */
-      EMPATHY_INDIVIDUAL_FEATURE_CHAT |
-      EMPATHY_INDIVIDUAL_FEATURE_CALL |
-      EMPATHY_INDIVIDUAL_FEATURE_EDIT |
-      EMPATHY_INDIVIDUAL_FEATURE_INFO |
-      EMPATHY_INDIVIDUAL_FEATURE_LOG |
-      EMPATHY_INDIVIDUAL_FEATURE_SMS |
-      EMPATHY_INDIVIDUAL_FEATURE_CALL_PHONE);
+  gtk_widget_show (GTK_WIDGET (self->priv->view));
 
-  gtk_widget_show (GTK_WIDGET (self->priv->individual_view));
-  gtk_container_add (GTK_CONTAINER (sw),
-      GTK_WIDGET (self->priv->individual_view));
-  g_signal_connect (self->priv->individual_view, "row-activated",
-     G_CALLBACK (roster_window_row_activated_cb), self);
+  egg_list_box_add_to_scrolled (EGG_LIST_BOX (self->priv->view),
+      GTK_SCROLLED_WINDOW (sw));
+
+  g_signal_connect (self->priv->view, "individual-activated",
+      G_CALLBACK (individual_activated_cb), self);
+  g_signal_connect (self->priv->view, "event-activated",
+      G_CALLBACK (event_activated_cb), self);
+  g_signal_connect (self->priv->view, "popup-individual-menu",
+      G_CALLBACK (popup_individual_menu_cb), self);
+  g_signal_connect (self->priv->view, "notify::empty",
+      G_CALLBACK (view_empty_cb), self);
+  g_signal_connect (self->priv->view, "individual-tooltip",
+      G_CALLBACK (individual_tooltip_cb), self);
+
+  gtk_widget_set_has_tooltip (GTK_WIDGET (self->priv->view), TRUE);
 
   /* Set up search bar */
   self->priv->search_bar = empathy_live_search_new (
-      GTK_WIDGET (self->priv->individual_view));
-  empathy_individual_view_set_live_search (self->priv->individual_view,
+      GTK_WIDGET (self->priv->view));
+  empathy_roster_view_set_live_search (self->priv->view,
       EMPATHY_LIVE_SEARCH (self->priv->search_bar));
   gtk_box_pack_start (GTK_BOX (search_vbox), self->priv->search_bar,
       FALSE, TRUE, 0);
 
   g_signal_connect_swapped (self, "map",
-      G_CALLBACK (gtk_widget_grab_focus), self->priv->individual_view);
-
-  /* Connect to proper signals to check if contact list is empty or not */
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (self->priv->individual_view));
-  self->priv->empty = TRUE;
-  g_signal_connect (model, "row-inserted",
-      G_CALLBACK (roster_window_row_inserted_cb), self);
-  g_signal_connect (model, "row-deleted",
-      G_CALLBACK (roster_window_row_deleted_cb), self);
+      G_CALLBACK (gtk_widget_grab_focus), self->priv->view);
 
   /* Load user-defined accelerators. */
   roster_window_accels_load ();
@@ -2315,10 +2114,11 @@ empathy_roster_window_init (EmpathyRosterWindow *self)
   self->priv->call_observer = empathy_call_observer_dup_singleton ();
   self->priv->event_manager = empathy_event_manager_dup_singleton ();
 
-  g_signal_connect (self->priv->event_manager, "event-added",
-      G_CALLBACK (roster_window_event_added_cb), self);
-  g_signal_connect (self->priv->event_manager, "event-removed",
-      G_CALLBACK (roster_window_event_removed_cb), self);
+  tp_g_signal_connect_object (self->priv->event_manager, "event-added",
+      G_CALLBACK (roster_window_event_added_cb), self, 0);
+  tp_g_signal_connect_object (self->priv->event_manager, "event-removed",
+      G_CALLBACK (roster_window_event_removed_cb), self, 0);
+
   g_signal_connect (self->priv->account_manager, "account-validity-changed",
       G_CALLBACK (roster_window_account_validity_changed_cb), self);
   g_signal_connect (self->priv->account_manager, "account-removed",
@@ -2327,22 +2127,10 @@ empathy_roster_window_init (EmpathyRosterWindow *self)
       G_CALLBACK (roster_window_account_disabled_cb), self);
 
   g_settings_bind (self->priv->gsettings_ui, EMPATHY_PREFS_UI_SHOW_OFFLINE,
-      self->priv->individual_view, "show-offline",
-      G_SETTINGS_BIND_GET);
-  g_settings_bind (self->priv->gsettings_ui, EMPATHY_PREFS_UI_SHOW_PROTOCOLS,
-      self->priv->individual_store, "show-protocols",
-      G_SETTINGS_BIND_GET);
-  g_settings_bind (self->priv->gsettings_ui, EMPATHY_PREFS_UI_COMPACT_CONTACT_LIST,
-      self->priv->individual_store, "is-compact",
-      G_SETTINGS_BIND_GET);
-  g_settings_bind (self->priv->gsettings_ui, EMPATHY_PREFS_UI_SHOW_AVATARS,
-      self->priv->individual_store, "show-avatars",
-      G_SETTINGS_BIND_GET);
-  g_settings_bind (self->priv->gsettings_contacts, EMPATHY_PREFS_CONTACTS_SORT_CRITERIUM,
-      self->priv->individual_store, "sort-criterium",
+      self->priv->view, "show-offline",
       G_SETTINGS_BIND_GET);
   g_settings_bind (self->priv->gsettings_ui, EMPATHY_PREFS_UI_SHOW_GROUPS,
-      self->priv->individual_store, "show-groups",
+      self->priv->view, "show-groups",
       G_SETTINGS_BIND_GET);
   g_settings_bind (self->priv->gsettings_ui, "show-balance-in-roster",
       self->priv->balance_vbox, "visible",
